#include <xsTypes.h>
#include <functionMap.h>
#include <stlToCArrays.h>
#include <XSUtil/Utils/XSutility.h>
#include <cfortran.h>

// Functions called FROM here to Fortran:

PROTOCCALLSFSUB2(INITNEI,initnei,PINT,PINT)
#define INITNEI(nionp,nzmax) CCALLSFSUB2(INITNEI,initnei,PINT,PINT,nionp,nzmax)

PROTOCCALLSFSUB10(DOXSNEQS,doxsneqs,FLOATV,INT,FLOATV,INT,FLOATV,FLOATV,INTV,INTV,INT,INT)
#define DOXSNEQS(ear,ne,param,ifl,photar,ei,ionel,ionstage,nionp,nzmax) \
           CCALLSFSUB10(DOXSNEQS,doxsneqs,FLOATV,INT,FLOATV,INT,FLOATV,FLOATV,INTV,INTV,INT,INT, \
             ear,ne,param,ifl,photar,ei,ionel,ionstage,nionp,nzmax)



void xneqs(const RealArray& energyArray, const RealArray& params,
        int spectrumNumber, RealArray& flux, RealArray& fluxErr, 
        const string& initString)
{
   //    Aug 23,99 subroutine hunt used instead of ibine for distribution of
   //              lines into bins.
   //    Aug 9,99  Nonequlibrium ionization models with constant electron
   //              temperature, with a distribution of ionization timescales
   //              appropriate for a plane parallel shock or its section.
   //		   additive model for XSPEC to use with general models.
   //		   parameters (param):
   //              (1):    temperature t(keV)
   //	           (2):    heavy metal abundance (with respect to solar)
   //                    ionization range bounded by:
   //              (3):    ionization parameter taul(cm^-3 s)
   //              (4):    ionization parameter tauu(cm^-3 s)
   //		   (5):	   redshift z
   
   // Translated from xsneqs.f 03/09 (CG)
   
   RealArray vparam(17);
   vparam[0] = params[0];
   vparam[1] = 1.0;
   vparam[2] = 1.0;
   for (size_t i=3; i<14; ++i)
      vparam[i] = params[1];
   vparam[14] = params[2];
   vparam[15] = params[3];
   vparam[16] = params[4];
   
   // Call the xsneqs subroutine for variable abundances.
   
   xsneqs(energyArray, vparam, spectrumNumber, flux, fluxErr, initString);
}


void xsneqs(const RealArray& energyArray, const RealArray& params,
        int spectrumNumber, RealArray& flux, RealArray& fluxErr, 
        const string& initString)
{
   // Wrapper function for memory allocation.  Actual work is performed
   // in doxsneqs.f
   
   using namespace XSutility;
   
   static bool isFirst = true;
   static int nionp=0;
   static int nzmax=0;
   static auto_array_ptr<float> apEi(0);
   static auto_array_ptr<int> apIonel(0);
   static auto_array_ptr<int> apIonstage(0);
   
   if (isFirst)
   {
      INITNEI(nionp, nzmax);
      apEi.reset(new float[nionp]);
      apIonel.reset(new int[nionp]);
      apIonstage.reset(new int[nionp]);
      isFirst = false;
   }
   
   int ne = static_cast<int>(energyArray.size()-1);
   float *ear=0, *pars=0, *photar=0, *photer=0;
   XSFunctions::stlToFloatArrays<float>(energyArray, params, flux, fluxErr,
           ear, pars, photar, photer);
   auto_array_ptr<float> apEar(ear);
   auto_array_ptr<float> apPars(pars);
   auto_array_ptr<float> apPhotar(photar);
   auto_array_ptr<float> apPhoter(photer);
       
   DOXSNEQS(ear, ne, pars, spectrumNumber, photar, apEi.get(),
          apIonel.get(), apIonstage.get(), nionp, nzmax);
      
   XSFunctions::floatFluxToStl<float>(photar, photer, ne, false, flux, fluxErr);       
}
